#ベイズ統計モデリング

#データの読み込み
DF <- read.table(file="defective.csv", sep=",", header=T)

#製品の不良発生数 : defective（目的変数 y）
#製品の製造時の温度(測定値) : temperature（説明変数 x）

#要約統計量を表示
summary(DF)
#散布図を表示
plot(DF$temperature, DF$defective, col="steelblue")

#通常の線形回帰モデルを当てはめた場合
LM <- lm(defective ~ temperature, data=DF)
summary(LM)


#ライブラリの読み込み
library(rstan)

#観測値をリスト化する（名称をx, yとする）
D <- list(N=nrow(DF), 
          x=DF$temperature, y=DF$defective)

#モデリングを実行
#モデル：別ファイルのスクリプト bayesian.stan
#データ：リスト化された観測値（x, y）
#seedは乱数系列の指定（通常は省略してよい）
fit <- stan(file="4.4.03.bayesian.stan", data=D, seed=123)
fit

#シミュレーション結果(1回ごとの値)を取り出す
exlist <- rstan::extract(fit)

#回帰係数b0, b1の期待値
mean(exlist$b0)
mean(exlist$b1)

#回帰係数の分布を図示（ヒストグラム）
hist(exlist$b0, breaks=100, col="steelblue")
hist(exlist$b1, breaks=100, col="steelblue")

#回帰係数b0, b1の95％確信区間
#quantile()は指定された累積比で値を取り出す関数
#この例では下から2.5％と97.5％に相当する値を表示
quantile(exlist$b0, probs=c(0.025, 0.975))
quantile(exlist$b1, probs=c(0.025, 0.975))

#温度の推定値（期待値と誤差）
z_mean <- apply(exlist$z, 2, mean)
z_sd   <- apply(exlist$z, 2, sd)


#散布図でモデルを視覚化

#横軸に温度の観測値、縦軸に不良品発生件（▲）
plot(D$x, D$y, 
     pch=17, col=alpha("darkblue",0.8), 
     xlim=c(20,60), ylim=c(80,220))

#横軸に温度の推定値、縦軸に不良品発生件（●）
par(new=T)  #重ね書きの指定
plot(z_mean, D$y, 
     pch=19, col=alpha("orange",0.6), 
     xlim=c(20,60), ylim=c(80,220), ann=F)

#回帰直線
par(new=T)
plot(function(x){mean(exlist$b0)+mean(exlist$b1)*x}, 
     col=alpha("orange",0.8), 
     xlim=c(20,60), ylim=c(80,220), ann=F)

#縦軸の推定値に誤差(±sd)の線分を加える
#arrows()は矢印を描く関数
# codeは矢じりの形の指定、angleは矢じりの角度の指定
arrows(z_mean-z_sd, D$y, 
       z_mean+z_sd, D$y, 
       col=alpha("orange",0.6), 
       length=.02, code=3, angle=90)
