# coding: utf-8

# ## 決定境界を表示するプログラム


# ライブラリの読み込み
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt


# ### 他のプログラムから呼び出して使える関数を定義


# 関数 plot_bound() を定義
# 引数 features : 特徴量（2次元のndarrayまたはデータフレーム）
#      model : フィッティング済みのモデル（分類器）
#      levels : 水準数（ラベルの水準数を指定、省略時は自動で決定）
#      interaction : 交互作用の有無（省略時は False）
#      steps : グリッドの分割数（省略時は512×512）
#      cmap : matplotlib のカラーマップ（省略時は viridis）
#      fill : 背景色を塗りつぶすかどうか（省略時は True）
#      alpha : 透明度（省略時は 0.1）

def plot_bound(features, model, levels=None, interaction=False, 
               steps=512, cmap="viridis", fill=True, alpha=0.15):
    
    # 特徴量ベクトルがndarrayの場合はデータフレームに変換する
    if isinstance(features, np.ndarray) :
        features = pd.DataFrame(features)

    # 各特徴量（0列目,1列目）の最小・最大値
    x0_min = min(features.iloc[:, 0])
    x0_max = max(features.iloc[:, 0])
    x1_min = min(features.iloc[:, 1])
    x1_max = max(features.iloc[:, 1])
    
    #　最小・最大値にマージンを加える（描画範囲を広めにとるため）
    x0_margin = (x0_max - x0_min)/20
    x1_margin = (x1_max - x1_min)/20
    x0_min = x0_min - x0_margin
    x0_max = x0_max + x0_margin
    x1_min = x1_min - x1_margin
    x1_max = x1_max + x1_margin

    # 横軸と縦軸のそれぞれを最小値から最大値までの範囲をstepsの数で分割し、
    # グリッドの各ポジション(格子の交点)の座標を取得
    # x0pos ：steps×stepsのグリッドの横軸の座標
    # x1pos ：steps×stepsのグリッドの縦軸の座標
    # x0pos, x1posはともに512×512の2次元のndarray形式
    x0pos, x1pos = np.mgrid[x0_min:x0_max:(steps*1j), x1_min:x1_max:(steps*1j)]
    
    # ポジションの座標を格納したndarrayの形式を2次元から1次元に変換
    x0 = x0pos.ravel()
    x1 = x1pos.ravel()
   
    # 横軸の座標x0と、縦軸の座標x1を列結合(2列のndarray形式)
    if interaction == True:
        # 交互作用ありの場合は3列目をx0とx1の積とする
        x01 = np.c_[x0, x1, x0*x1]
    else :
        # 通常の場合はx0とx1を1列目と2列目にして結合
        x01 = np.c_[x0, x1]

    # グリッド上の座標の位置に対応する予測値を取得
    pred = model.predict(x01)
    
    # 予測値のベクトルを2次元に変換（x0posの行数・列数に合わせる）
    pred = pred.reshape(x0pos.shape)
    
    # 描き分ける水準数から1を引いて境界線の数とする
    if levels is not None:
        levels = levels - 1

    # 塗りつぶしの有無で対応する関数を呼び出す
    if fill == True :
        # 等高線を描く関数（塗りつぶしあり）
        #   等高線の基準となる高さとして予測値 pred を指定する
        #   levelsは等高線の数（Noneの場合は自動で決定される）
        plt.contourf(x0pos, x1pos, pred, 
                 levels=levels, cmap=cmap, alpha=alpha)
        # 等高線を描く関数（塗りつぶしなし）
        plt.contour(x0pos, x1pos, pred, linewidths=2.0,
                 levels=levels, cmap=cmap, alpha=alpha)
    else :       
        # 等高線を描く関数（塗りつぶしなし）
        plt.contour(x0pos, x1pos, pred, linewidths=2.0,
                 levels=levels, cmap=cmap, alpha=alpha)
        
    #　描画範囲を設定
    plt.xlim(x0_min, x0_max)
    plt.ylim(x1_min, x1_max)


# ### 関数の利用例
# 
# 以下の部分は、このプログラムを直接実行する場合にだけ動作させることを前提とします。  
# plot_bound.py として他のプログラムから呼び出して実行する場合には動作しない箇所です。


# 直接実行されているかの判断
if __name__ == '__main__':            
    
    # ライブラリから関数を読み込む（決定木）
    from sklearn.tree import DecisionTreeClassifier
    
    # CSVファイルのデータ（カンマ区切り）をデータフレーム形式で読み込み
    df = pd.read_csv("machine_status.csv", sep=',')
    
    # 特徴量を取り出す
    X = df[["Noise","RPM"]]
    y = df["Check"]
    
    # モデルのオブジェクトを作成
    clf = DecisionTreeClassifier(max_depth=2)
    # モデルをデータに適合させる(フィッティング)
    clf.fit(X, y)
    
    # 描画の大きさ
    plt.figure(figsize=(6, 5))   
    # 散布図を作成（s=は点の大きさ、c=は色の指定）
    plt.scatter(X["Noise"], X["RPM"], c=df["Check"], 
                s=40, cmap="winter_r", alpha=0.9)
    # 定義した関数を呼び出して境界線と背景色を追加
    plot_bound(X, clf, cmap="winter_r") 
    # 表題を追加（ハイパーパラメータの値をモデルから取得）
    plt.title(str(type(clf)) + 
              " max_depth: " + str(clf.max_depth) +
              " min_samples_leaf: " + str(clf.min_samples_leaf) )
    # 軸ラベルを追加
    plt.xlabel("Noise")
    plt.ylabel("RPM")   
    # 表示
    plt.show()


